/*******************************************************************************
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 * 
 * Contact Info:
 * 	Bruce Donald
 * 	Duke University
 * 	Department of Computer Science
 * 	Levine Science Research Center (LSRC)
 * 	Durham
 * 	NC 27708-0129 
 * 	USA
 * 	brd@cs.duke.edu
 * 
 * Copyright (C) 2011 Jeffrey W. Martin and Bruce R. Donald
 * 
 * <signature of Bruce Donald>, April 2011
 * Bruce Donald, Professor of Computer Science
 ******************************************************************************/


package edu.duke.donaldLab.share.protein;

import java.util.ArrayList;
import java.util.List;

import edu.duke.donaldLab.share.util.Bijection;
import edu.duke.donaldLab.share.util.HashBijection;

public class SubunitOrder
{
	/**************************
	 *   Data Members
	 **************************/
	
	private String m_referenceOrder;
	
	
	/**************************
	 *   Constructors
	 **************************/
	
	public SubunitOrder( String referenceOrder )
	{
		m_referenceOrder = referenceOrder;
	}
	
	
	/**************************
	 *   Accessors
	 **************************/
	
	public int getNumSubunits( )
	{
		return m_referenceOrder.length();
	}
	
	
	/**************************
	 *   Methods
	 **************************/
	
	public String toString( )
	{
		return m_referenceOrder;
	}
	
	public void convertComputedToReference( Protein protein )
	{
		// build a bijection between computed subunit ids (keys) and reference subunit ids (values)
		Bijection<Integer,Integer> bijection = new HashBijection<Integer,Integer>();
		for( int i=0; i<m_referenceOrder.length(); i++ )
		{
			bijection.put( i, protein.getSubunitId( m_referenceOrder.charAt( i ) ) );
		}
		
		// perform the mapping
		List<Subunit> subunits = protein.getSubunits();
		List<Subunit> newSubunits = new ArrayList<Subunit>( subunits.size() );
		for( int i=0; i<subunits.size(); i++ )
		{
			newSubunits.add( null );
		}
		for( Bijection.Entry<Integer,Integer> entry : bijection.entries() )
		{
			int computedId = entry.getKey();
			int referenceId = entry.getValue();
			newSubunits.set( referenceId, subunits.get( computedId ) );
		}
		
		protein.setSubunits( newSubunits );
	}
}
